@@ -6,7 +6,9 @@ from urllib.parse import urlencode, quote
 from .browser import browser, get_playwright
 from playwright import sync_playwright
 import logging
+import os
 from .utilities import update_messager
+os.environ['no_proxy'] = '127.0.0.1,localhost'
 
 BASE_URL = "https://m.tiktok.com/"
 
@@ -34,9 +36,12 @@ class TikTokApi:
             "Chrome/86.0.4240.111 Safari/537.36"
         )
         self.proxy = kwargs.get("proxy", None)
-        if kwargs.get("persistent_browser", True):
+
+        self.signer_url = kwargs.get("external_signer", None)
+        if self.signer_url == None:
             self.browser = browser(**kwargs)
             self.userAgent = self.browser.userAgent
+        
 
         try:
             self.timezone_name = self.__format_new_params__(self.browser.timezone_name)
@@ -85,7 +90,22 @@ class TikTokApi:
             pass
         TikTokApi.__instance = None
 
-    def getData(self, b, **kwargs) -> dict:
+    def external_signer(self, url, custom_did=None):
+        if custom_did != None:
+            query = {
+                    "url": url,
+                    "custom_did": custom_did
+            }
+        else:
+            query = {
+                    "url": url,
+            }
+        data = requests.get(self.signer_url + "?{}".format(urlencode(query)))
+        parsed_data = data.json()
+
+        return parsed_data['verifyFp'], parsed_data['did'], parsed_data['_signature'], parsed_data['userAgent'], parsed_data['referrer']
+
+    def getData(self, **kwargs) -> dict:
         """Returns a dictionary of a response from TikTok.
 
         :param api_url: the base string without a signature
@@ -102,14 +122,21 @@ class TikTokApi:
             language,
             proxy,
             maxCount,
+            did,
         ) = self.__process_kwargs__(kwargs)
+        kwargs['custom_did'] = did
         if self.request_delay is not None:
             time.sleep(self.request_delay)
 
         if self.proxy != None:
             proxy = self.proxy
 
-        verify_fp, did, signature = self.browser.sign_url(**kwargs)
+        if self.signer_url == None:
+            verify_fp, did, signature = self.browser.sign_url(**kwargs)
+            userAgent = self.browser.userAgent
+            referrer = self.browser.referrer
+        else:
+            verify_fp, did, signature, userAgent, referrer = self.external_signer(kwargs['url'], custom_did=kwargs.get('custom_did', None))
         query = {"verifyFp": verify_fp, "did": did, "_signature": signature}
         url = "{}&{}".format(kwargs["url"], urlencode(query))
         r = requests.get(
@@ -122,11 +149,11 @@ class TikTokApi:
                 "accept": "application/json, text/plain, */*",
                 "accept-encoding": "gzip, deflate, br",
                 "accept-language": "en-US,en;q=0.9",
-                "referer": self.browser.referrer,
+                "referer": referrer,
                 "sec-fetch-dest": "empty",
                 "sec-fetch-mode": "cors",
                 "sec-fetch-site": "same-site",
-                "user-agent": self.browser.userAgent,
+                "user-agent": userAgent,
                 "cookie": "tt_webid_v2=" + did,
             },
             proxies=self.__format_proxy(proxy),
@@ -141,7 +168,7 @@ class TikTokApi:
             logging.info(r.text)
             raise Exception("Invalid Response")
 
-    def getBytes(self, b, **kwargs) -> bytes:
+    def getBytes(self, **kwargs) -> bytes:
         """Returns bytes of a response from TikTok.
 
         :param api_url: the base string without a signature
@@ -158,8 +185,15 @@ class TikTokApi:
             language,
             proxy,
             maxCount,
+            did,
         ) = self.__process_kwargs__(kwargs)
-        verify_fp, did, signature = b.sign_url(**kwargs)
+        kwargs['custom_did'] = did
+        if self.signer_url == None:
+            verify_fp, did, signature = self.browser.sign_url(**kwargs)
+            userAgent = self.browser.userAgent
+            referrer = self.browser.referrer
+        else:
+            verify_fp, did, signature, userAgent, referrer = self.external_signer(kwargs['url'], custom_did=kwargs.get('custom_did', None))
         query = {"verifyFp": verify_fp, "_signature": signature}
         url = "{}&{}".format(kwargs["url"], urlencode(query))
         r = requests.get(
@@ -175,7 +209,7 @@ class TikTokApi:
                 "Pragma": "no-cache",
                 "Range": "bytes=0-",
                 "Referer": "https://www.tiktok.com/",
-                "User-Agent": b.userAgent,
+                "User-Agent": userAgent,
             },
             proxies=self.__format_proxy(proxy),
         )
@@ -190,7 +224,9 @@ class TikTokApi:
             language,
             proxy,
             maxCount,
+            did,
         ) = self.__process_kwargs__(kwargs)
+        kwargs['custom_did'] = did
 
         response = []
         first = True
@@ -216,7 +252,7 @@ class TikTokApi:
             api_url = "{}api/item_list/?{}&{}".format(
                 BASE_URL, self.__add_new_params__(), urlencode(query)
             )
-            res = self.getData(self.browser, url=api_url, **kwargs)
+            res = self.getData(url=api_url, **kwargs)
 
             for t in res.get("items", []):
                 response.append(t)
@@ -227,6 +263,7 @@ class TikTokApi:
 
             realCount = count - len(response)
             maxCursor = res["maxCursor"]
+            minCursor = res['minCursor']
 
             first = False
 
@@ -274,7 +311,9 @@ class TikTokApi:
             language,
             proxy,
             maxCount,
+            did,
         ) = self.__process_kwargs__(kwargs)
+        kwargs['custom_did'] = did
 
         response = []
         offsetCount = 0
@@ -291,7 +330,7 @@ class TikTokApi:
             api_url = "{}api/discover/{}/?{}&{}".format(
                 BASE_URL, prefix, self.__add_new_params__(), urlencode(query)
             )
-            data = self.getData(self.browser, url=api_url, **kwargs)
+            data = self.getData(url=api_url, **kwargs)
 
             if "userInfoList" in data.keys():
                 for x in data["userInfoList"]:
@@ -330,7 +369,9 @@ class TikTokApi:
             language,
             proxy,
             maxCount,
+            did,
         ) = self.__process_kwargs__(kwargs)
+        kwargs['custom_did'] = did
 
         response = []
         first = True
@@ -358,7 +399,7 @@ class TikTokApi:
                 BASE_URL, self.__add_new_params__(), urlencode(query)
             )
 
-            res = self.getData(self.browser, url=api_url, **kwargs)
+            res = self.getData(url=api_url, **kwargs)
             if "items" in res.keys():
                 for t in res["items"]:
                     response.append(t)
@@ -391,7 +432,9 @@ class TikTokApi:
             language,
             proxy,
             maxCount,
+            did,
         ) = self.__process_kwargs__(kwargs)
+        kwargs['custom_did'] = did
         data = self.getUserObject(username, **kwargs)
         return self.userPosts(
             data["id"],
@@ -421,7 +464,9 @@ class TikTokApi:
             language,
             proxy,
             maxCount,
+            did,
         ) = self.__process_kwargs__(kwargs)
+        kwargs['custom_did'] = did
 
         api_url = "https://m.tiktok.com/api/item_list/?{}&count={}&id={}&type=1&secUid={}" "&minCursor={}&maxCursor={}&sourceType=8&appId=1233&region={}&language={}".format(
             self.__add_new_params__(),
@@ -434,7 +479,7 @@ class TikTokApi:
             language,
         )
 
-        return self.getData(self.browser, url=api_url, **kwargs)
+        return self.getData(url=api_url, **kwargs)
 
     def getUserPager(self, username, page_size=30, minCursor=0, maxCursor=0, **kwargs):
         """Returns a generator to page through a user's feed
@@ -454,7 +499,9 @@ class TikTokApi:
             language,
             proxy,
             maxCount,
+            did,
         ) = self.__process_kwargs__(kwargs)
+        kwargs['custom_did'] = did
         data = self.getUserObject(username, **kwargs)
 
         while True:
@@ -501,7 +548,9 @@ class TikTokApi:
             language,
             proxy,
             maxCount,
+            did,
         ) = self.__process_kwargs__(kwargs)
+        kwargs['custom_did'] = did
         response = []
         first = True
 
@@ -528,7 +577,7 @@ class TikTokApi:
                 BASE_URL, self.__add_new_params__(), urlencode(query)
             )
 
-            res = self.getData(self.browser, url=api_url, **kwargs)
+            res = self.getData(url=api_url, **kwargs)
 
             try:
                 res["items"]
@@ -569,7 +618,9 @@ class TikTokApi:
             language,
             proxy,
             maxCount,
+            did,
         ) = self.__process_kwargs__(kwargs)
+        kwargs['custom_did'] = did
         data = self.getUserObject(username, **kwargs)
         return self.userLiked(
             data["id"],
@@ -596,7 +647,9 @@ class TikTokApi:
             language,
             proxy,
             maxCount,
+            did,
         ) = self.__process_kwargs__(kwargs)
+        kwargs['custom_did'] = did
         response = []
 
         while len(response) < count:
@@ -617,7 +670,7 @@ class TikTokApi:
                 BASE_URL, self.__add_new_params__(), urlencode(query)
             )
 
-            res = self.getData(self.browser, url=api_url, **kwargs)
+            res = self.getData(url=api_url, **kwargs)
 
             for t in res.get("itemList", []):
                 response.append(t)
@@ -644,14 +697,16 @@ class TikTokApi:
             language,
             proxy,
             maxCount,
+            did,
         ) = self.__process_kwargs__(kwargs)
+        kwargs['custom_did'] = did
 
         query = {"musicId": id, "language": language}
         api_url = "{}api/music/detail/?{}&{}".format(
             BASE_URL, self.__add_new_params__(), urlencode(query)
         )
 
-        return self.getData(self.browser, url=api_url, **kwargs)
+        return self.getData(url=api_url, **kwargs)
 
     def byHashtag(self, hashtag, count=30, offset=0, **kwargs) -> dict:
         """Returns a dictionary listing TikToks with a specific hashtag.
@@ -670,7 +725,9 @@ class TikTokApi:
             language,
             proxy,
             maxCount,
+            did,
         ) = self.__process_kwargs__(kwargs)
+        kwargs['custom_did'] = did
         id = self.getHashtagObject(hashtag)["challengeInfo"]["challenge"]["id"]
         response = []
 
@@ -691,7 +748,7 @@ class TikTokApi:
             api_url = "{}api/challenge/item_list/?{}&{}".format(
                 BASE_URL, self.__add_new_params__(), urlencode(query)
             )
-            res = self.getData(self.browser, url=api_url, **kwargs)
+            res = self.getData(url=api_url, **kwargs)
 
             for t in res["itemList"]:
                 response.append(t)
@@ -717,12 +774,14 @@ class TikTokApi:
             language,
             proxy,
             maxCount,
+            did,
         ) = self.__process_kwargs__(kwargs)
+        kwargs['custom_did'] = did
         query = {"challengeName": hashtag, "language": language}
         api_url = "{}api/challenge/detail/?{}&{}".format(
             BASE_URL, self.__add_new_params__(), urlencode(query)
         )
-        return self.getData(self.browser, url=api_url, **kwargs)
+        return self.getData(url=api_url, **kwargs)
 
     def getHashtagDetails(self, hashtag, **kwargs) -> dict:
         """Returns a hashtag object.
@@ -737,13 +796,15 @@ class TikTokApi:
             language,
             proxy,
             maxCount,
+            did,
         ) = self.__process_kwargs__(kwargs)
+        kwargs['custom_did'] = did
         query = {"language": language}
         api_url = "{}node/share/tag/{}?{}&{}".format(
             BASE_URL, quote(hashtag), self.__add_new_params__(), urlencode(query)
         )
 
-        return self.getData(self.browser, url=api_url, **kwargs)
+        return self.getData(url=api_url, **kwargs)
 
     def getRecommendedTikToksByVideoID(
         self, id, count=30, minCursor=0, maxCursor=0, **kwargs
@@ -760,7 +821,9 @@ class TikTokApi:
             language,
             proxy,
             maxCount,
+            did,
         ) = self.__process_kwargs__(kwargs)
+        kwargs['custom_did'] = did
 
         response = []
         first = True
@@ -787,7 +850,7 @@ class TikTokApi:
                 BASE_URL, self.__add_new_params__(), urlencode(query)
             )
 
-            res = self.getData(self.browser, url=api_url, **kwargs)
+            res = self.getData(url=api_url, **kwargs)
 
             for t in res.get("items", []):
                 response.append(t)
@@ -816,7 +879,9 @@ class TikTokApi:
             language,
             proxy,
             maxCount,
+            did,
         ) = self.__process_kwargs__(kwargs)
+        kwargs['custom_did'] = did
         did = kwargs.get("custom_did", None)
         query = {
             "itemId": id,
@@ -826,7 +891,7 @@ class TikTokApi:
             BASE_URL, self.__add_new_params__(), urlencode(query)
         )
 
-        return self.getData(self.browser, url=api_url, **kwargs)
+        return self.getData(url=api_url, **kwargs)
 
     def getTikTokByUrl(self, url, **kwargs) -> dict:
         """Returns a dictionary of a TikTok object by url.
@@ -841,7 +906,9 @@ class TikTokApi:
             language,
             proxy,
             maxCount,
+            did,
         ) = self.__process_kwargs__(kwargs)
+        kwargs['custom_did'] = did
         custom_did = kwargs.get("custom_did", None)
         if "@" in url and "/video/" in url:
             post_id = url.split("/video/")[1].split("?")[0]
@@ -866,13 +933,15 @@ class TikTokApi:
             language,
             proxy,
             maxCount,
+            did,
         ) = self.__process_kwargs__(kwargs)
+        kwargs['custom_did'] = did
         query = {"noUser": 1, "userCount": 30, "scene": 0}
         api_url = "{}node/share/discover?{}&{}".format(
             BASE_URL, self.__add_new_params__(), urlencode(query)
         )
 
-        return self.getData(self.browser, url=api_url, **kwargs)["body"][1][
+        return self.getData(url=api_url, **kwargs)["body"][1][
             "exploreList"
         ]
 
@@ -886,13 +955,15 @@ class TikTokApi:
             language,
             proxy,
             maxCount,
+            did,
         ) = self.__process_kwargs__(kwargs)
+        kwargs['custom_did'] = did
         query = {"noUser": 1, "userCount": 30, "scene": 0}
         api_url = "{}node/share/discover?{}&{}".format(
             BASE_URL, self.__add_new_params__(), urlencode(query)
         )
 
-        return self.getData(self.browser, url=api_url, **kwargs)["body"][2][
+        return self.getData(url=api_url, **kwargs)["body"][2][
             "exploreList"
         ]
 
@@ -909,7 +980,9 @@ class TikTokApi:
             language,
             proxy,
             maxCount,
+            did,
         ) = self.__process_kwargs__(kwargs)
+        kwargs['custom_did'] = did
         return self.getUser(username, **kwargs)["user"]
 
     def getUser(self, username, **kwargs) -> dict:
@@ -925,13 +998,15 @@ class TikTokApi:
             language,
             proxy,
             maxCount,
+            did,
         ) = self.__process_kwargs__(kwargs)
+        kwargs['custom_did'] = did
         query = {"uniqueId": username, "language": language}
         api_url = "{}api/user/detail/?{}&{}".format(
             BASE_URL, self.__add_new_params__(), urlencode(query)
         )
 
-        return self.getData(self.browser, url=api_url, **kwargs)["userInfo"]
+        return self.getData(url=api_url, **kwargs)["userInfo"]
 
     def getSuggestedUsersbyID(
         self, userId="6745191554350760966", count=30, **kwargs
@@ -947,7 +1022,9 @@ class TikTokApi:
             language,
             proxy,
             maxCount,
+            did,
         ) = self.__process_kwargs__(kwargs)
+        kwargs['custom_did'] = did
         query = {
             "noUser": 0,
             "pageId": userId,
@@ -960,7 +1037,7 @@ class TikTokApi:
         )
 
         res = []
-        for x in self.getData(self.browser, url=api_url, **kwargs)["body"][0][
+        for x in self.getData(url=api_url, **kwargs)["body"][0][
             "exploreList"
         ]:
             res.append(x["cardItem"])
@@ -981,7 +1058,9 @@ class TikTokApi:
             language,
             proxy,
             maxCount,
+            did,
         ) = self.__process_kwargs__(kwargs)
+        kwargs['custom_did'] = did
         users = []
         unusedIDS = [startingId]
         while len(users) < count:
@@ -1010,7 +1089,9 @@ class TikTokApi:
             language,
             proxy,
             maxCount,
+            did,
         ) = self.__process_kwargs__(kwargs)
+        kwargs['custom_did'] = did
         query = {
             "noUser": 0,
             "pageId": userId,
@@ -1023,7 +1104,7 @@ class TikTokApi:
         )
 
         res = []
-        for x in self.getData(self.browser, url=api_url, **kwargs)["body"][1][
+        for x in self.getData(url=api_url, **kwargs)["body"][1][
             "exploreList"
         ]:
             res.append(x["cardItem"])
@@ -1044,7 +1125,9 @@ class TikTokApi:
             language,
             proxy,
             maxCount,
+            did,
         ) = self.__process_kwargs__(kwargs)
+        kwargs['custom_did'] = did
         hashtags = []
         ids = self.getSuggestedUsersbyIDCrawler(
             count=count, startingId=startingId, **kwargs
@@ -1074,7 +1157,9 @@ class TikTokApi:
             language,
             proxy,
             maxCount,
+            did,
         ) = self.__process_kwargs__(kwargs)
+        kwargs['custom_did'] = did
         query = {
             "noUser": 0,
             "pageId": userId,
@@ -1087,7 +1172,7 @@ class TikTokApi:
         )
 
         res = []
-        for x in self.getData(self.browser, url=api_url, **kwargs)["body"][2][
+        for x in self.getData(url=api_url, **kwargs)["body"][2][
             "exploreList"
         ]:
             res.append(x["cardItem"])
@@ -1108,7 +1193,9 @@ class TikTokApi:
             language,
             proxy,
             maxCount,
+            did,
         ) = self.__process_kwargs__(kwargs)
+        kwargs['custom_did'] = did
         musics = []
         ids = self.getSuggestedUsersbyIDCrawler(
             count=count, startingId=startingId, **kwargs
@@ -1135,7 +1222,9 @@ class TikTokApi:
             language,
             proxy,
             maxCount,
+            did,
         ) = self.__process_kwargs__(kwargs)
+        kwargs['custom_did'] = did
         try:
             api_url = data["video"]["downloadAddr"]
         except Exception:
@@ -1156,8 +1245,10 @@ class TikTokApi:
             language,
             proxy,
             maxCount,
+            did,
         ) = self.__process_kwargs__(kwargs)
-        return self.getBytes(self.browser, url=download_url, **kwargs)
+        kwargs['custom_did'] = did
+        return self.getBytes(url=download_url, **kwargs)
 
     def get_Video_By_Url(self, video_url, **kwargs) -> bytes:
         (
@@ -1165,12 +1256,14 @@ class TikTokApi:
             language,
             proxy,
             maxCount,
+            did,
         ) = self.__process_kwargs__(kwargs)
+        kwargs['custom_did'] = did
 
         tiktok_schema = self.getTikTokByUrl(video_url, **kwargs)
         download_url = tiktok_schema["itemInfo"]["itemStruct"]["video"]["downloadAddr"]
 
-        return self.getBytes(self.browser, url=download_url, **kwargs)
+        return self.getBytes(url=download_url, **kwargs)
 
     def get_Video_No_Watermark(self, video_url, return_bytes=0, **kwargs) -> bytes:
         """Gets the video with no watermark
@@ -1184,7 +1277,9 @@ class TikTokApi:
             language,
             proxy,
             maxCount,
+            did,
         ) = self.__process_kwargs__(kwargs)
+        kwargs['custom_did'] = did
         r = requests.get(
             video_url,
             headers={
@@ -1292,5 +1387,7 @@ class TikTokApi:
         language = kwargs.get("language", "en")
         proxy = kwargs.get("proxy", None)
         maxCount = kwargs.get("maxCount", 35)
+        did = kwargs.get("did", str(random.randint(10000, 999999999)))
+        kwargs['did'] = did
 
-        return region, language, proxy, maxCount
+        return region, language, proxy, maxCount, did
